# Server -> Bridge
CALL_MESSAGE_TYPE = "FUNCTION_CALL"
GET_HOW_TO_SERVICE_REGISTRATION = "GET_HOW_TO_SERVICE_REGISTRATION"
REGISTRATION_MESSAGE = "REGISTRATION"
OAUTH_RETURN = "OAUTH_RETURN"
DATA_CALLBACK = "CALLBACK"

# Bridge -> Server
AUTHENTICATION = "AUTHENTICATION"
NOTIFICATION = "NOTIFICATION"
CONFIGURATION = "CONFIGURATION"
